IF EXISTS (SELECT *
           FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'CONSULTA_CSOSN' AND
	              TYPE = 'P' 			      AND
      	        U.UID = O.UID            AND
      	        U.NAME = USER)
DROP PROCEDURE CONSULTA_CSOSN
GO
CREATE PROCEDURE CONSULTA_CSOSN @P_ENTSAICSOSN  VARCHAR(01),
                                @P_EMPRESA     VARCHAR(02),
                                @P_CDARVPROD   VARCHAR(20),
                                @P_INSCESTA    VARCHAR(16),
                                @P_TIPOOPER    VARCHAR(02),
                                @P_FILIAL      VARCHAR(04),
                                @P_LANCTONF    VARCHAR(10),
                                @P_CDGRUPOFISC VARCHAR(03),
                                @P_IDCSOSNINTEREST VARCHAR(01),
                                @P_CDFISCOPERSN  VARCHAR(04) OUTPUT
AS
   DECLARE @V_CDARVPROD     VARCHAR(20);
   DECLARE @V_CONTADOR      INTEGER;
   DECLARE @V_IDINTEREST    VARCHAR(01);
   DECLARE @V_ORIGDESTOP    VARCHAR(01);
   DECLARE @V_EMPRESA       VARCHAR(02);
   DECLARE @V_INSCESTA      VARCHAR(16);
   DECLARE @V_CDFILIRECE    VARCHAR(04);
   DECLARE @V_CDFISCOPERSN  VARCHAR(03);
   DECLARE @V_CDCSTDIG1PROD VARCHAR(01);
   DECLARE @V_IDORIGDESTOP  VARCHAR(01);
   DECLARE @V_RESULT        VARCHAR(03);

BEGIN

   SET @V_CONTADOR = 0;
   SET @V_RESULT = 'X';
   SET @V_CDARVPROD = NULL;

----------------------------------------------------------------
/*BLOCO VERIFICA O PRIMEIRO DGITO DO CSOSN*/
----------------------------------------------------------------
   IF @P_ENTSAICSOSN = 'E' BEGIN /*OPERAES DE ENTRADA*/
      /*Verifica o 1 dgito no cadastro do produto*/
      SELECT @V_CDCSTDIG1PROD = MAX(CDCSTDIG1PROD)
      FROM PRODUTO
      WHERE (CDARVPROD = @P_CDARVPROD);

      /*Se no cadastro do produto for vazio verifica se o destino da nota  cliente/fornecedor/filial*/
      IF @V_CDCSTDIG1PROD IS NULL BEGIN
         SELECT @V_IDORIGDESTOP = MAX(IDORIGDESTOP)
         FROM TIPOOPERAC
         WHERE CDTIPOOPER   = @P_TIPOOPER AND
               IDENTRSAIDOP = @P_ENTSAICSOSN;
      END;

      IF @V_IDORIGDESTOP = 'C' BEGIN /*Quando o destino da nota for um cliente*/
         SELECT @V_CDCSTDIG1PROD = MAX(CDCSTDIG1CLI)
         FROM CLIENTE C LEFT JOIN NOTAFISCAL N
                        ON (C.CDCLIENTE = N.CDCLIENTE)
         WHERE N.CDFILIAL   = @P_FILIAL    AND
               N.NRLANCTONF = @P_LANCTONF;
      END
      ELSE BEGIN
         IF @V_IDORIGDESTOP = 'F' BEGIN /*Quando o destino da nota for um fornecedor*/
            SELECT @V_CDCSTDIG1PROD = MAX(CDCSTDIG1ENT)
            FROM FORNECEDOR F LEFT JOIN NOTAFISCAL N
                              ON (F.CDFORNECED = N.CDFORNECED)
            WHERE N.CDFILIAL   = @P_FILIAL AND
                  N.NRLANCTONF = @P_LANCTONF;
         END
         ELSE BEGIN /*Quando o destino da nota for uma filial*/
            SELECT @V_EMPRESA = MAX(CDEMPRESA),
                   @V_INSCESTA = MAX(CDINSCESTA)
            FROM FILIAL
            WHERE CDFILIAL IN (SELECT MAX(CDFILIRECE)
                               FROM NOTAFISCAL
                               WHERE CDFILIAL   = @P_FILIAL   AND
                                     NRLANCTONF = @P_LANCTONF);

            SELECT @V_CDCSTDIG1PROD = MAX(CDCSTDIG1SAI)
            FROM INSCRESTAD
            WHERE CDEMPRESA  = @V_EMPRESA AND
                  CDINSCESTA = @V_INSCESTA;
         END;
      END;
   END
   ELSE BEGIN /*OPERAES DE SADA*/
      /*Verifica o 1 dgito no cadastro do produto*/
      SELECT @V_CDCSTDIG1PROD = MAX(CDCSTDIG1PROD)
      FROM PRODUTO
      WHERE (CDARVPROD = @P_CDARVPROD);

      IF @V_CDCSTDIG1PROD IS NULL BEGIN
         SELECT @V_CDCSTDIG1PROD = MAX(CDCSTDIG1SAI)
           FROM INSCRESTAD
          WHERE (CDEMPRESA  = @P_EMPRESA) AND
                (CDINSCESTA = @P_INSCESTA);
      END;
   END;
----------------------------------------------------------------
/*FIM DO BLOCO QUE VERIFICA O PRIMEIRO DGITO DO CSOSN*/
----------------------------------------------------------------

   IF @P_IDCSOSNINTEREST = 'S' BEGIN
      IF (@P_CDGRUPOFISC IS NOT NULL) BEGIN
         IF @P_CDARVPROD IS NOT NULL BEGIN
            SELECT @V_CDARVPROD = MAX(P.CDARVPROD),
                   @V_CONTADOR  = COUNT(*)
            FROM CODFISCOPERSN C LEFT JOIN PRODUTO P ON
                                (C.CDPRODUTO   = P.CDPRODUTO)
            WHERE (IDENTSAICSOSN = @P_ENTSAICSOSN)
              AND (CDEMPRESA    = @P_EMPRESA)
              AND (CDTIPOOPER   = @P_TIPOOPER)
              AND (CDINSCESTA   = @P_INSCESTA)
              AND (CDGRUPOFISC  = @P_CDGRUPOFISC)
              AND (IDCSOSNINTEREST = 'S')
              AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),@P_CDARVPROD),@P_CDARVPROD) = 1);
         END
         ELSE BEGIN
            SELECT @V_CDARVPROD = MAX(C.CDPRODUTO),
                   @V_CONTADOR  = COUNT(*)
            FROM CODFISCOPERSN C
            WHERE (IDENTSAICSOSN = @P_ENTSAICSOSN)
              AND (CDEMPRESA    = @P_EMPRESA)
              AND (CDTIPOOPER   = @P_TIPOOPER)
              AND (CDINSCESTA   = @P_INSCESTA)
              AND (CDGRUPOFISC  = @P_CDGRUPOFISC)
              AND (IDCSOSNINTEREST = 'S')
              AND (CDPRODUTO IS NULL);
         END;
      END;

      IF (@V_CONTADOR > 0) BEGIN
         IF (NOT @V_CDARVPROD IS NULL) BEGIN
            SELECT @V_CDFISCOPERSN = ISNULL(MAX(CDFISCOPERSN), 'X')
              FROM CODFISCOPERSN C LEFT JOIN PRODUTO P ON
                                (C.CDPRODUTO    = P.CDPRODUTO)
             WHERE (C.IDENTSAICSOSN = @P_ENTSAICSOSN)
               AND (C.CDEMPRESA    = @P_EMPRESA)
               AND (C.CDTIPOOPER   = @P_TIPOOPER)
               AND (C.CDINSCESTA   = @P_INSCESTA)
               AND (C.CDGRUPOFISC  = @P_CDGRUPOFISC)
               AND (C.IDCSOSNINTEREST = 'S')
               AND (P.CDARVPROD    = @V_CDARVPROD);

            SET @V_RESULT = @V_CDFISCOPERSN;
         END
         ELSE IF (@V_CDARVPROD IS NULL) BEGIN
            SELECT @V_CDFISCOPERSN = ISNULL(MAX(CDFISCOPERSN), 'X')
              FROM CODFISCOPERSN C
             WHERE (C.IDENTSAICSOSN = @P_ENTSAICSOSN)
               AND (C.CDEMPRESA    = @P_EMPRESA)
               AND (C.CDTIPOOPER   = @P_TIPOOPER)
               AND (C.CDINSCESTA   = @P_INSCESTA)
               AND (C.CDGRUPOFISC  = @P_CDGRUPOFISC)
               AND (C.IDCSOSNINTEREST = 'S')
               AND (C.CDPRODUTO IS NULL);

            SET @V_RESULT = @V_CDFISCOPERSN;
         END;
      END
      ELSE BEGIN
         IF @P_CDARVPROD IS NOT NULL BEGIN
            SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
              FROM CODFISCOPERSN C LEFT JOIN PRODUTO P ON
                                  (C.CDPRODUTO = P.CDPRODUTO)
             WHERE (IDENTSAICSOSN = @P_ENTSAICSOSN)
               AND (CDEMPRESA    = @P_EMPRESA)
               AND (CDTIPOOPER   = @P_TIPOOPER)
               AND (CDINSCESTA   = @P_INSCESTA)
               AND (CDGRUPOFISC IS NULL)
               AND (IDCSOSNINTEREST = 'S')
               AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),@P_CDARVPROD),@P_CDARVPROD) = 1);
         END
         ELSE BEGIN
            SELECT @V_CDARVPROD = MAX(C.CDPRODUTO)
              FROM CODFISCOPERSN C
             WHERE (IDENTSAICSOSN = @P_ENTSAICSOSN)
               AND (CDEMPRESA    = @P_EMPRESA)
               AND (CDTIPOOPER   = @P_TIPOOPER)
               AND (CDINSCESTA   = @P_INSCESTA)
               AND (CDGRUPOFISC IS NULL)
               AND (IDCSOSNINTEREST = 'S')
               AND (C.CDPRODUTO IS NULL);
         END;
         IF NOT (@V_CDARVPROD IS NULL) BEGIN
            SELECT @V_CDFISCOPERSN = ISNULL(MAX(CDFISCOPERSN), 'X')
              FROM CODFISCOPERSN C LEFT JOIN PRODUTO P ON
                                 (C.CDPRODUTO = P.CDPRODUTO)
             WHERE (C.IDENTSAICSOSN = @P_ENTSAICSOSN)
               AND (C.CDEMPRESA    = @P_EMPRESA)
               AND (C.CDTIPOOPER   = @P_TIPOOPER)
               AND (C.CDINSCESTA   = @P_INSCESTA)
               AND (C.CDGRUPOFISC IS NULL)
               AND (C.IDCSOSNINTEREST = 'S')
               AND (P.CDARVPROD    = @V_CDARVPROD);

            SET @V_RESULT = @V_CDFISCOPERSN;
         END
         ELSE IF (@V_CDARVPROD IS NULL) BEGIN
            SELECT @V_CDFISCOPERSN = ISNULL(MAX(CDFISCOPERSN), 'X')
              FROM CODFISCOPERSN C
             WHERE (C.IDENTSAICSOSN = @P_ENTSAICSOSN)
               AND (C.CDEMPRESA    = @P_EMPRESA)
               AND (C.CDTIPOOPER   = @P_TIPOOPER)
               AND (C.CDINSCESTA   = @P_INSCESTA)
               AND (C.CDGRUPOFISC IS NULL)
               AND (C.IDCSOSNINTEREST = 'S')
               AND (C.CDPRODUTO IS NULL);
            SET @V_RESULT = @V_CDFISCOPERSN;
         END;
      END;
   END;

   IF @V_RESULT = 'X' BEGIN
      IF (@P_CDGRUPOFISC IS NOT NULL) BEGIN
         IF @P_CDARVPROD IS NOT NULL BEGIN
            SELECT @V_CDARVPROD = MAX(P.CDARVPROD),
                   @V_CONTADOR  = COUNT(*)
              FROM CODFISCOPERSN C LEFT JOIN PRODUTO P ON
                              (C.CDPRODUTO   = P.CDPRODUTO)
             WHERE (IDENTSAICSOSN = @P_ENTSAICSOSN)
               AND (CDEMPRESA    = @P_EMPRESA)
               AND (CDTIPOOPER   = @P_TIPOOPER)
               AND (CDINSCESTA   = @P_INSCESTA)
               AND (CDGRUPOFISC  = @P_CDGRUPOFISC)
               AND (IDCSOSNINTEREST = 'N')
               AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),@P_CDARVPROD),@P_CDARVPROD) = 1);
         END
         ELSE BEGIN
            SELECT @V_CDARVPROD = MAX(C.CDPRODUTO),
                   @V_CONTADOR  = COUNT(*)
              FROM CODFISCOPERSN C
             WHERE (IDENTSAICSOSN = @P_ENTSAICSOSN)
               AND (CDEMPRESA    = @P_EMPRESA)
               AND (CDTIPOOPER   = @P_TIPOOPER)
               AND (CDINSCESTA   = @P_INSCESTA)
               AND (CDGRUPOFISC  = @P_CDGRUPOFISC)
               AND (IDCSOSNINTEREST = 'N')
               AND (CDPRODUTO IS NULL);
         END;

         IF (@V_CONTADOR > 0) BEGIN
            IF (NOT @V_CDARVPROD IS NULL) BEGIN
               SELECT @V_CDFISCOPERSN = MAX(CDFISCOPERSN)
                 FROM CODFISCOPERSN C LEFT JOIN PRODUTO P ON
                                     (C.CDPRODUTO    = P.CDPRODUTO)
                WHERE (C.IDENTSAICSOSN = @P_ENTSAICSOSN)
                  AND (C.CDEMPRESA    = @P_EMPRESA)
                  AND (C.CDTIPOOPER   = @P_TIPOOPER)
                  AND (C.CDINSCESTA   = @P_INSCESTA)
                  AND (C.CDGRUPOFISC  = @P_CDGRUPOFISC)
                  AND (C.IDCSOSNINTEREST = 'N')
                  AND (P.CDARVPROD    = @V_CDARVPROD);
               SET @V_RESULT = @V_CDFISCOPERSN;
            END
            ELSE IF (@V_CDARVPROD IS NULL) BEGIN
               SELECT @V_CDFISCOPERSN = MAX(CDFISCOPERSN)
                 FROM CODFISCOPERSN C
                WHERE (C.IDENTSAICSOSN = @P_ENTSAICSOSN)
                  AND (C.CDEMPRESA    = @P_EMPRESA)
                  AND (C.CDTIPOOPER   = @P_TIPOOPER)
                  AND (C.CDINSCESTA   = @P_INSCESTA)
                  AND (C.CDGRUPOFISC  = @P_CDGRUPOFISC)
                  AND (C.IDCSOSNINTEREST = 'N')
                  AND (C.CDPRODUTO IS NULL);

               SET @V_RESULT = @V_CDFISCOPERSN;
            END;
         END;
      END
      ELSE BEGIN
         IF @P_CDARVPROD IS NOT NULL BEGIN
            SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
            FROM CODFISCOPERSN C LEFT JOIN PRODUTO P ON
                              (C.CDPRODUTO = P.CDPRODUTO)
            WHERE (IDENTSAICSOSN = @P_ENTSAICSOSN)
              AND (CDEMPRESA    = @P_EMPRESA)
              AND (CDTIPOOPER   = @P_TIPOOPER)
              AND (CDINSCESTA   = @P_INSCESTA)
              AND (CDGRUPOFISC IS NULL)
              AND (IDCSOSNINTEREST = 'N')
              AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),@P_CDARVPROD),@P_CDARVPROD) = 1);
         END
         ELSE BEGIN
            SELECT @V_CDARVPROD = MAX(C.CDPRODUTO)
            FROM CODFISCOPERSN C
            WHERE (IDENTSAICSOSN = @P_ENTSAICSOSN)
              AND (CDEMPRESA    = @P_EMPRESA)
              AND (CDTIPOOPER   = @P_TIPOOPER)
              AND (CDINSCESTA   = @P_INSCESTA)
              AND (CDGRUPOFISC IS NULL)
              AND (IDCSOSNINTEREST = 'N')
              AND (C.CDPRODUTO IS NULL);
         END;
         IF NOT (@V_CDARVPROD IS NULL) BEGIN
            SELECT @V_CDFISCOPERSN = MAX(CDFISCOPERSN)
              FROM CODFISCOPERSN C LEFT JOIN PRODUTO P ON
                                   (C.CDPRODUTO = P.CDPRODUTO)
             WHERE (C.IDENTSAICSOSN = @P_ENTSAICSOSN)
               AND (C.CDEMPRESA    = @P_EMPRESA)
               AND (C.CDTIPOOPER   = @P_TIPOOPER)
               AND (C.CDINSCESTA   = @P_INSCESTA)
               AND (C.CDGRUPOFISC IS NULL)
               AND (C.IDCSOSNINTEREST = 'N')
               AND (P.CDARVPROD    = @V_CDARVPROD);

            SET @V_RESULT = @V_CDFISCOPERSN;
         END
         ELSE IF (@V_CDARVPROD IS NULL) BEGIN
            SELECT @V_CDFISCOPERSN = MAX(CDFISCOPERSN)
              FROM CODFISCOPERSN C
             WHERE (C.IDENTSAICSOSN = @P_ENTSAICSOSN)
               AND (C.CDEMPRESA    = @P_EMPRESA)
               AND (C.CDTIPOOPER   = @P_TIPOOPER)
               AND (C.CDINSCESTA   = @P_INSCESTA)
               AND (C.CDGRUPOFISC IS NULL)
               AND (C.IDCSOSNINTEREST = 'N')
               AND (C.CDPRODUTO IS NULL);

            SET @V_RESULT = @V_CDFISCOPERSN;
         END;
      END;
   END;
   SET @P_CDFISCOPERSN = @V_CDCSTDIG1PROD+@V_RESULT;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'CONSULTA_CSOSN','5.45.xxx';
GO

